@@ -33,18 +33,18 @@ import numpy as np
 import scipy.sparse as sp
 from scipy import linalg
 
-from .base import (
+from sklearn.base import (
     BaseEstimator,
     ClassNamePrefixFeaturesOutMixin,
     TransformerMixin,
     _fit_context,
 )
-from .exceptions import DataDimensionalityWarning
-from .utils import check_random_state
-from .utils._param_validation import Interval, StrOptions, validate_params
-from .utils.extmath import safe_sparse_dot
-from .utils.random import sample_without_replacement
-from .utils.validation import check_array, check_is_fitted, validate_data
+from sklearn.exceptions import DataDimensionalityWarning
+from sklearn.utils import check_random_state
+from sklearn.utils._param_validation import Interval, StrOptions, validate_params
+from sklearn.utils.extmath import safe_sparse_dot
+from sklearn.utils.random import sample_without_replacement
+from sklearn.utils.validation import check_array, check_is_fitted, validate_data
 
 __all__ = [
     "GaussianRandomProjection",
