@@ -19,17 +19,15 @@ from sklearn.base import (
     clone,
     is_classifier,
     is_clusterer,
-    is_outlier_detector,
     is_regressor,
 )
 from sklearn.cluster import KMeans
 from sklearn.decomposition import PCA
-from sklearn.ensemble import IsolationForest
 from sklearn.exceptions import InconsistentVersionWarning
 from sklearn.metrics import get_scorer
 from sklearn.model_selection import GridSearchCV, KFold
 from sklearn.pipeline import Pipeline
-from sklearn.preprocessing import StandardScaler
+from sklearn.preprocessing import LabelEncoder, StandardScaler
 from sklearn.svm import SVC, SVR
 from sklearn.tree import DecisionTreeClassifier, DecisionTreeRegressor
 from sklearn.utils._mocking import MockDataFrame
@@ -239,6 +237,22 @@ def test_clone_class_rather_than_instance():
         clone(MyEstimator)
 
 
+def test_conditional_attrs_not_in_dir():
+    # Test that __dir__ includes only relevant attributes. #28558
+
+    encoder = LabelEncoder()
+    assert "set_output" not in dir(encoder)
+
+    scalar = StandardScaler()
+    assert "set_output" in dir(scalar)
+
+    svc = SVC(probability=False)
+    assert "predict_proba" not in dir(svc)
+
+    svc.probability = True
+    assert "predict_proba" in dir(svc)
+
+
 def test_repr():
     # Smoke test the repr of the base estimator.
     my_estimator = MyEstimator()
@@ -269,21 +283,6 @@ def test_get_params():
         test.set_params(a__a=2)
 
 
-# TODO(1.8): Remove this test when the deprecation is removed
-def test_is_estimator_type_class():
-    with pytest.warns(FutureWarning, match="passing a class to.*is deprecated"):
-        assert is_classifier(SVC)
-
-    with pytest.warns(FutureWarning, match="passing a class to.*is deprecated"):
-        assert is_regressor(SVR)
-
-    with pytest.warns(FutureWarning, match="passing a class to.*is deprecated"):
-        assert is_clusterer(KMeans)
-
-    with pytest.warns(FutureWarning, match="passing a class to.*is deprecated"):
-        assert is_outlier_detector(IsolationForest)
-
-
 @pytest.mark.parametrize(
     "estimator, expected_result",
     [
@@ -394,6 +393,7 @@ def test_set_params_updates_valid_params():
     ],
 )
 def test_score_sample_weight(tree, dataset):
+    tree = clone(tree)  # avoid side effects from previous tests.
     rng = np.random.RandomState(0)
     # check that the score with and without sample weights are different
     X, y = dataset
@@ -560,6 +560,8 @@ def test_pickle_version_warning_is_issued_when_no_version_info_in_pickle():
         pickle.loads(tree_pickle_noversion)
 
 
+# The test modifies global state by changing the TreeNoVersion class
+@pytest.mark.thread_unsafe
 def test_pickle_version_no_warning_is_issued_with_non_sklearn_estimator():
     iris = datasets.load_iris()
     tree = TreeNoVersion().fit(iris.data, iris.target)
@@ -758,7 +760,7 @@ def test_feature_names_in():
     with pytest.raises(ValueError, match=msg):
         trans.transform(df_bad)
 
-    # warns when fitted on dataframe and transforming a ndarray
+    # warns when fitted on dataframe and transforming an ndarray
     msg = (
         "X does not have valid feature names, but NoOpTransformer was "
         "fitted with feature names"
@@ -766,7 +768,7 @@ def test_feature_names_in():
     with pytest.warns(UserWarning, match=msg):
         trans.transform(X_np)
 
-    # warns when fitted on a ndarray and transforming dataframe
+    # warns when fitted on an ndarray and transforming dataframe
     msg = "X has feature names, but NoOpTransformer was fitted without feature names"
     trans = NoOpTransformer().fit(X_np)
     with pytest.warns(UserWarning, match=msg):
@@ -1051,6 +1053,19 @@ def test_param_is_non_default(default_value, test_value):
     assert "param" in non_default
 
 
+def test_param_is_non_default_when_pandas_NA():
+    """Check that we detect pandas.Na as non-default parameter.
+
+    Non-regression test for:
+    https://github.com/scikit-learn/scikit-learn/issues/32312
+    """
+    pd = pytest.importorskip("pandas")
+
+    estimator = make_estimator_with_param(default_value=0)(param=pd.NA)
+    non_default = estimator._get_params_html().non_default
+    assert "param" in non_default
+
+
 @pytest.mark.parametrize(
     "default_value, test_value",
     [
